package net.wanji.utc.task;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dto.CrossSchemeRingsDTO;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.CrossSchemeRings;
import net.wanji.feign.service.common.FeignCommon;
import net.wanji.utc.cache.CrossInfoCache;
import net.wanji.utc.cache.UtcFeignClientCache;
import net.wanji.utc.common.typeenum.HisenseTurnEnum;
import net.wanji.utc.common.typeenum.HiseseDirEnum;
import net.wanji.utc.dto.ring.RingPhaseInfoDTO;
import net.wanji.utc.util.CommonUtils;
import net.wanji.utc.util.RedisUtil;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2025/03/26 10:00
 * @description 每十分钟同步海信信号机方案环图请求
 */
@Component
@Slf4j
@Profile("docker")
public class HisenseSchemeRingTask {

    @Resource
    private RedisUtil redisUtil;
    @Resource
    private CrossInfoCache crossInfoCache;
    @Resource
    private UtcFeignClientCache utcFeignClientCache;

    @Scheduled(fixedRate = 10 * 60 * 1000, initialDelay = 30 * 1000)
    public void getSchemeRing() throws Exception {
        List<CrossInfoPO> crossInfoList = CrossInfoCache.crossInfoList;
        ObjectMapper mapper = JacksonUtils.getInstance();
        if (!CollectionUtils.isEmpty(crossInfoList)) {
            List<CrossInfoPO> collect = crossInfoList.stream().filter(po -> Objects.equals(22, po.getManufacturerId())).collect(Collectors.toList());
            for (CrossInfoPO crossInfoPO : collect) {
                String crossId = crossInfoPO.getId();
                String manufacturerCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
                FeignCommon utcFeignClientService = utcFeignClientCache.getUtcFeignClientService(manufacturerCode);
                CrossSchemeRingsDTO crossSchemeRingsDTO = new CrossSchemeRingsDTO();
                crossSchemeRingsDTO.setCrossId(crossId);
                JsonViewObject jsonViewObject = null;
                try {
                    jsonViewObject = utcFeignClientService.crossSchemeRings(crossSchemeRingsDTO);
                    if (Objects.nonNull(jsonViewObject) && Objects.equals(200, jsonViewObject.getCode())) {
                        Object content = jsonViewObject.getContent();
                        String contentStr = mapper.writeValueAsString(content);
                        CrossSchemeRings crossSchemeRings = mapper.convertValue(jsonViewObject.getContent(), CrossSchemeRings.class);
                        if (Objects.nonNull(crossSchemeRings)) {
                            String code = crossSchemeRings.getSpot();
                            String schemeNo = crossSchemeRings.getPattern();
                            String cycleLen = crossSchemeRings.getCycleLen();
                            String offset = crossSchemeRings.getOffset();
                            String ringCount = crossSchemeRings.getRingCount();
                            JSONObject cycleList = crossSchemeRings.getCycleList();
                            // 中间容器 环号-相位号列表
                            Map<Integer, List<Integer>> ringPhaseMap = new HashMap<>();
                            for (Map.Entry<String, Object> jsonEntry : cycleList.entrySet()) {
                                String ringNoStr = jsonEntry.getKey();
                                String ringNo = "";
                                String value = (String) jsonEntry.getValue();
                                List<Integer> phaseList = new ArrayList<>();
                                if (StringUtils.isNotBlank(ringNoStr) && StringUtils.isNotBlank(value)) {
                                    ringNo = ringNoStr.replaceAll("Cycle", "");
                                    String tempStr = value.replaceAll("_", " ");
                                    String[] tempList = tempStr.split(" ");
                                    for (String s : tempList) {
                                        phaseList.add(Integer.valueOf(s));
                                    }
                                }
                                ringPhaseMap.put(Integer.valueOf(ringNo), phaseList);
                            }
                            List<CrossSchemeRings.Phase> phaseList = crossSchemeRings.getPhaseList();
                            if (!CollectionUtils.isEmpty(phaseList)) {
                                List<Integer> greenTimeList = new ArrayList<>();
                                List<Integer> yellowTimeList = new ArrayList<>();
                                List<Integer> redTimeList = new ArrayList<>();
                                List<Integer> maxGTimeList = new ArrayList<>();
                                List<Integer> minGTimeList = new ArrayList<>();
                                List<Integer> directionList = new ArrayList<>();
                                List<Integer> channelDimList = new ArrayList<>();
                                List<Integer> concurrentList = new ArrayList<>();
                                for (CrossSchemeRings.Phase phase : phaseList) {
                                    String greenTimeStr = phase.getGreenTime();
                                    String yellowTimeStr = phase.getYellowTime();
                                    String redTimeStr = phase.getRedTime();
                                    String maxGTimeStr = phase.getMaxGTime();
                                    String minGTimeStr = phase.getMinGTime();
                                    //String directionStr = phase.getDirection();
                                    String channelDimStr = phase.getChannelDim();
                                    String concurrentStr = phase.getConcurrent();
                                    greenTimeList = getListInt(greenTimeStr);
                                    yellowTimeList = getListInt(yellowTimeStr);
                                    redTimeList = getListInt(redTimeStr);
                                    maxGTimeList = getListInt(maxGTimeStr);
                                    minGTimeList = getListInt(minGTimeStr);
                                    //directionList = getListInt(directionStr);
                                    channelDimList = getListInt(channelDimStr);
                                    concurrentList = getListInt(concurrentStr);
                                }
                                List<RingPhaseInfoDTO> ringPhaseInfoDTOS = new ArrayList<RingPhaseInfoDTO>(16);
                                for (int i = 0; i < 16; i++) {
                                    RingPhaseInfoDTO ringPhaseInfoDTO = new RingPhaseInfoDTO();
                                    Integer phaseId = i + 1;
                                    ringPhaseInfoDTO.setPhaseId(phaseId);
                                    Integer curRingNo = 0;
                                    Integer sort = -1;
                                    for (Map.Entry<Integer, List<Integer>> entry : ringPhaseMap.entrySet()) {
                                        List<Integer> value = entry.getValue();
                                        if (value.contains(phaseId)) {
                                            sort  = value.indexOf(phaseId);
                                            curRingNo = entry.getKey();
                                        }
                                    }
                                    if (!Objects.equals(-1, sort)) {
                                        ringPhaseInfoDTO.setSort(sort + 1);
                                    }
                                    ringPhaseInfoDTO.setSchemeNo((Integer.valueOf(schemeNo) + 2) / 3);
                                    ringPhaseInfoDTO.setOffset(Integer.valueOf(offset));
                                    ringPhaseInfoDTO.setCycleLen(Integer.valueOf(cycleLen));
                                    ringPhaseInfoDTO.setRingNo(curRingNo);
                                    Integer greenTime = greenTimeList.get(i);
                                    Integer yellowTime = yellowTimeList.get(i);
                                    Integer redTime = redTimeList.get(i);
                                    ringPhaseInfoDTO.setPhaseTime(greenTime + redTime + yellowTime);
                                    ringPhaseInfoDTO.setGreenTime(greenTime);
                                    ringPhaseInfoDTO.setYellowTime(yellowTime);
                                    ringPhaseInfoDTO.setRedTime(redTime);
                                    ringPhaseInfoDTO.setMaxGTime(maxGTimeList.get(i));
                                    ringPhaseInfoDTO.setMinGTime(minGTimeList.get(i));
                                    //ringPhaseInfoDTO.setDirection(directionList.get(i));
                                    Integer channel = channelDimList.get(i);
                                    String binaryString = Long.toBinaryString(channel);
                                    //获取方向转向分组
                                    List<String> binaryList = CommonUtils.groupStringFromRight(binaryString);
                                    List<RingPhaseInfoDTO.DirTurn> dirTurnList = new ArrayList<>();
                                    for (String readUtf8Line : binaryList) {
                                        RingPhaseInfoDTO.DirTurn dirTurn = new RingPhaseInfoDTO.DirTurn();
                                        // 高三位 方向
                                        String dir = readUtf8Line.substring(0, 3);
                                        // 低五位 转向
                                        String turn = readUtf8Line.substring(3);
                                        Integer wjLightsDir = HiseseDirEnum.getWjLightsDir(Integer.parseInt(dir, 2));
                                        Integer wjLightsTurn = HisenseTurnEnum.getWjLightsTurn(Integer.parseInt(turn, 2));
                                        // 不需要行人，不需要右转
                                        if (Objects.equals(0, wjLightsDir) || Objects.equals(0, wjLightsTurn)
                                                || Objects.equals(20, wjLightsTurn) || Objects.equals(3, wjLightsTurn)){
                                            continue;
                                        }
                                        dirTurn.setDir(wjLightsDir);
                                        dirTurn.setTurn(wjLightsTurn);
                                        dirTurnList.add(dirTurn);
                                    }
                                    ringPhaseInfoDTO.setDirTurnInfoList(dirTurnList);
                                    ringPhaseInfoDTO.setChannel(channel);
                                    ringPhaseInfoDTO.setConcurrent(concurrentList.get(i));
                                    ringPhaseInfoDTO.setDate(new Date());
                                    ringPhaseInfoDTOS.add(ringPhaseInfoDTO);
                                }
                                log.info("路口编号：{},当前环相位列表：{}", crossId, mapper.writeValueAsString(ringPhaseInfoDTOS));
                                redisUtil.setHash("utc-hisense-ring-phase", crossId, mapper.writeValueAsString(ringPhaseInfoDTOS));
                            }
                        }
                    } else {
                        log.error("海信环图请求异常：{}", jsonViewObject);
                    }
                } catch (Exception e) {
                    log.error("远程调用海信环图请求失败：{}", e);
                }
            }
        }
    }

    private List<Integer> getListInt(String str) {
        List<Integer> list = new ArrayList<>();
        if (StringUtils.isNotBlank(str)) {
            String[] s = str.split(" ");
            for (String s1 : s) {
                list.add(Integer.valueOf(s1));
            }
        }
        return list;
    }
}
